/*
 * RenderThread.java
 *
 * Created on 1 de noviembre de 2005, 22:32
 */

package Auto;
import com.threed.jpct.*;
import com.threed.jpct.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 *
 * @author  Gold Oozaru
 * @version 
 */
public class RenderThread extends Thread{
    static boolean OGL;

    World mundo;
    FrameBuffer buffer;
    KeyMapper mapaTeclas;
    KeyState estadoTeclas;
    TextureManager manejadorTexturas;
    Texture texturaActual;
    
    boolean izquierda;
    boolean derecha;
    boolean adelante;
    boolean atras;
    boolean luces;
    boolean salir;
    
    Object3D atrasO;
    Object3D adelanteO;
    Object3D otros;
    Object3D abajo;
    Object3D medio;
    Object3D vidrios;
    Object3D aros;
    Object3D base;
    Object3D capo;
    
    Object3D carro;
    
    Canvas canvas;
    /** Creates new RenderThread */
    public RenderThread(World am, FrameBuffer ab, KeyMapper ak, Canvas ac) {
        OGL=false;
        
        mundo=am;
        buffer=ab;
        mapaTeclas=ak;
        canvas=ac;
        
        texturas ();
        
        vidrios=new Object3D (0);
        capo=new Object3D (0);
        atrasO=new Object3D (0);
        adelanteO=new Object3D (0);
        medio=new Object3D (0);
        abajo=new Object3D (0);
        otros=new Object3D (0);
        aros=new Object3D (0);
        
        carro=new Object3D (0);
        
        cargar ("CARROGO.3DS", 0.1f);
        
        carro.addChild(atrasO);
        carro.addChild(adelanteO);
        carro.addChild(medio);
        carro.addChild(abajo);
        carro.addChild(otros);
        carro.addChild(vidrios);
        carro.addChild(aros);
        carro.addChild(capo);
        
        setEnv (atrasO);
        setEnv (adelanteO);
        setEnv (medio);
        setEnv (abajo);
        setEnv (capo);

        vidrios.setTransparency(1);
        /*
        base=cargar ("Base.3DS", 0.5f);
        base.translate (0, 3.5f, 0);*/
        mundo.addObject (atrasO);
        mundo.addObject (adelanteO);
        mundo.addObject (medio);
        mundo.addObject (otros);
        mundo.addObject (aros);
        mundo.addObject (capo);
        mundo.addObject (abajo);
        mundo.addObject (vidrios);
        
        aros.setAdditionalColor(new Color (20, 20, 30));
        mundo.buildAllObjects ();
    }
    
    void setEnv (Object3D O) {
    	O.setEnvmapped(Object3D.ENVMAP_ENABLED);
        O.setEnvmapMode(Object3D.ENVMAP_WORLDSPACE);
    }
        
    public void cargar (String fileName, float escala) {
		try {
	  
	        Object3D partes []=Loader.load3DS (getPath ()+"3DS/"+fileName, escala);
	        for (int i=0;i<partes.length;i++) {
	            Object3D part=partes [i];
	         
	            part.setCenter(SimpleVector.ORIGIN);
	            part.rotateX((float)-Math.PI/2);
	            part.rotateMesh();
	            part.setRotationMatrix(new Matrix());
	            if ((i&1)==1) part.setTransparency(0);
	            
	            if (part.getName ().startsWith ("ventana") || part.getName ().startsWith ("vidrio") || part.getName ().startsWith ("parabrisa")) {
	                vidrios=Object3D.mergeObjects (vidrios, part);
	            } else if (part.getName ().startsWith ("capo")) {
	            	capo=Object3D.mergeObjects (capo, part);
	            	
	            } else if (part.getName ().startsWith ("Aro") || part.getName ().startsWith ("eje")) {
	            	aros=Object3D.mergeObjects (aros, part);
	            	
	            } else if (part.getName ().startsWith ("medio") || part.getName ().startsWith ("frente") || part.getName ().startsWith ("cubreespe")) {
	            	medio=Object3D.mergeObjects (medio, part);
	            } else if (part.getName ().startsWith ("atras")) {
	            	atrasO=Object3D.mergeObjects (atrasO, part);
	            } else if (part.getName ().startsWith ("adelante")) {
	            	adelanteO=Object3D.mergeObjects (adelanteO, part);
	            } else if (part.getName ().startsWith ("abajo")) {
	            	abajo=Object3D.mergeObjects (abajo, part);
	            }
	            else {
	                otros=Object3D.mergeObjects (otros, part);
	            }
	        }
		} catch (Exception E) {
			E.printStackTrace();
		}
    }
    
    public void run () {
        
        while (!salir) {
            try {
                this.sleep(20);
            } catch (Exception E) {
                this.stop ();
            }
            
            buffer.clear ();
            buffer.update ();
            mundo.renderScene (buffer);
            mundo.draw (buffer);
            if (OGL) buffer.displayGLOnly ();
            canvas.repaint ();
            
            estadoTeclas=null;
            
            do {
                estadoTeclas=mapaTeclas.poll();
                if (estadoTeclas!=KeyState.NONE) {
                    obtenerEventos (estadoTeclas);
                }
            } while (estadoTeclas!=KeyState.NONE);
            
            hacerMovimientos ();
        }
        
        buffer.disableRenderer(IRenderer.RENDERER_OPENGL);
        this.stop ();
    }
    
    public void hacerMovimientos () {
        if (izquierda) {
            moverIzquierda ();
        }
        
        if (derecha) {
            moverDerecha ();
        }
        
        if (atras) {
            Camera c=mundo.getCamera ();
            c.moveCamera (Camera.CAMERA_MOVEOUT, 1f);
        }
        
        if (adelante) {
            Camera c=mundo.getCamera ();
            c.moveCamera (Camera.CAMERA_MOVEIN, 1f);
        }
    }
    
    public void obtenerEventos (KeyState evento) {
        boolean estado=evento.getState ();
    
        if (estadoTeclas.getKeyCode ()==KeyEvent.VK_UP)
            adelante=estado;
        
        if (estadoTeclas.getKeyCode ()==KeyEvent.VK_DOWN)
            atras=estado;
        
        if (estadoTeclas.getKeyCode ()==KeyEvent.VK_LEFT)
            izquierda=estado;
        
        if (estadoTeclas.getKeyCode ()==KeyEvent.VK_RIGHT)
            derecha=estado;
        
        if (estadoTeclas.getKeyCode ()==KeyEvent.VK_ESCAPE)
            salir=estado;
        
        if (estadoTeclas.getKeyCode ()==KeyEvent.VK_SPACE)
            luces=estado;
    }

    public String getPath () {
        System.out.println (this.getClass ().getResource (""));
        return ((this.getClass ().getResource ("").getFile())).replace ("%20", " ");
    }
    
    public void texturas () {

    	manejadorTexturas=TextureManager.getInstance ();
    	String pathTextura=getPath ()+"Texturas/";
    	File dir=new File (pathTextura); 

    	String texturas []=dir.list ();
    	for (int i=0;i<texturas.length;i++) {
    		manejadorTexturas.addTexture (""+texturas [i], new Texture (pathTextura+texturas [i]));
    	}
    }
    
    public void cambiarTextura (String parte) {
        
        if (parte.compareTo ("Base")==0) {
            manejadorTexturas.replaceTexture ("ATRAS.JPG", texturaActual);
            TextureInfo tis=new TextureInfo(manejadorTexturas.getTextureID("ATRAS.JPG"));
            tis.add(manejadorTexturas.getTextureID("ENV.JPG"), TextureInfo.MODE_ADD); // Or another mode...
            atrasO.setTexture(tis);
            adelanteO.setTexture(tis);
            medio.setTexture(tis);
            abajo.setTexture(tis);
            capo.setTexture(tis);
        }
    }
    
    public void moverIzquierda () {/*
        Camera temp=mundo.getCamera ();
        float radio=temp.getPosition ().length ();
        temp.setPosition (0, 0, 0);
        temp.rotateY (-0.02f);
        temp.moveCamera (Camera.CAMERA_MOVEOUT, radio);*/
    	carro.rotateY (-0.02f);
    }
    
    public void moverDerecha () {/*
        Camera temp=mundo.getCamera ();
        float radio=temp.getPosition ().length ();
        temp.setPosition (0, 0, 0);
        temp.rotateY (0.02f);
        temp.moveCamera (Camera.CAMERA_MOVEOUT, radio);*/
    	carro.rotateY (0.02f);
    }
}
